@@ -30,6 +30,7 @@
 #include "mrn_sys.hpp"
 #include "mrn_table.hpp"
 #include "mrn_mysql_compat.h"
+#include <mrn_lock.hpp>
 
 #if MYSQL_VERSION_ID >= 50603 && !defined(MRN_MARIADB_P)
 #  define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), TRUE)
@@ -51,6 +52,8 @@ extern "C" {
 
 extern HASH mrn_open_tables;
 extern pthread_mutex_t mrn_open_tables_mutex;
+extern HASH mrn_long_term_share;
+extern pthread_mutex_t mrn_long_term_share_mutex;
 extern char *mrn_default_parser;
 extern char *mrn_default_wrapper_engine;
 extern handlerton *mrn_hton_ptr;
@@ -386,25 +389,14 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
 
         switch (title_length)
         {
-        case 0:
-          continue;
         case 6:
           MRN_PARAM_STR("engine", engine);
-          error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-          my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-                          MYF(0), tmp_ptr);
-          goto error;
+          break;
         case 17:
           MRN_PARAM_STR("default_tokenizer", default_tokenizer);
-          error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-          my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-                          MYF(0), tmp_ptr);
-          goto error;
+          break;
         default:
-          error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-          my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-                          MYF(0), tmp_ptr);
-          goto error;
+          break;
         }
       }
     }
@@ -533,34 +525,14 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
 
     switch (title_length)
     {
-      case 0:
-        continue;
       case 5:
         MRN_PARAM_STR_LIST("table", index_table, i);
-        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-          MYF(0), tmp_ptr);
-        goto error;
+        break;
       case 6:
         MRN_PARAM_STR_LIST("parser", key_parser, i);
-        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-          MYF(0), tmp_ptr);
-        goto error;
-      case 10:
-        if (strncasecmp("normalizer", tmp_ptr, title_length) == 0) {
-          break;
-        } else {
-          error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-          my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-            MYF(0), tmp_ptr);
-          goto error;
-        }
+        break;
       default:
-        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-          MYF(0), tmp_ptr);
-        goto error;
+        break;
     }
   }
 #endif
@@ -618,8 +590,13 @@ int mrn_add_column_param(MRN_SHARE *share, Field *field, int i)
   int title_length;
   char *sprit_ptr[2];
   char *tmp_ptr, *start_ptr;
+
   MRN_DBUG_ENTER_FUNCTION();
 
+  if (share->wrapper_mode) {
+    DBUG_RETURN(0);
+  }
+
   DBUG_PRINT("info", ("mroonga create comment string"));
   if (
     !(param_string = my_strndup(field->comment.str,
@@ -661,25 +638,14 @@ int mrn_add_column_param(MRN_SHARE *share, Field *field, int i)
 
     switch (title_length)
     {
-      case 0:
-        continue;
       case 4:
         MRN_PARAM_STR_LIST("type", col_type, i);
-        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-          MYF(0), tmp_ptr);
-        goto error;
+        break;
       case 5:
         MRN_PARAM_STR_LIST("flags", col_flags, i);
-        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-          MYF(0), tmp_ptr);
-        goto error;
+        break;
       default:
-        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
-        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-          MYF(0), tmp_ptr);
-        goto error;
+        break;
     }
   }
 
@@ -722,6 +688,8 @@ int mrn_free_share_alloc(
   MRN_DBUG_ENTER_FUNCTION();
   if (share->engine)
     my_free(share->engine, MYF(0));
+  if (share->default_tokenizer)
+    my_free(share->default_tokenizer, MYF(0));
   for (i = 0; i < share->table_share->keys; i++)
   {
     if (share->index_table && share->index_table[i])
@@ -739,6 +707,65 @@ int mrn_free_share_alloc(
   DBUG_RETURN(0);
 }
 
+void mrn_free_long_term_share(MRN_LONG_TERM_SHARE *long_term_share)
+{
+  MRN_DBUG_ENTER_FUNCTION();
+  {
+    mrn::Lock lock(&mrn_long_term_share_mutex);
+    my_hash_delete(&mrn_long_term_share, (uchar*) long_term_share);
+  }
+  pthread_mutex_destroy(&long_term_share->auto_inc_mutex);
+  my_free(long_term_share, MYF(0));
+  DBUG_VOID_RETURN;
+}
+
+MRN_LONG_TERM_SHARE *mrn_get_long_term_share(const char *table_name,
+                                             uint table_name_length,
+                                             int *error)
+{
+  MRN_LONG_TERM_SHARE *long_term_share;
+  char *tmp_name;
+  MRN_DBUG_ENTER_FUNCTION();
+  DBUG_PRINT("info", ("mroonga: table_name=%s", table_name));
+  mrn::Lock lock(&mrn_long_term_share_mutex);
+  if (!(long_term_share = (MRN_LONG_TERM_SHARE*)
+    my_hash_search(&mrn_long_term_share, (uchar*) table_name,
+                   table_name_length)))
+  {
+    if (!(long_term_share = (MRN_LONG_TERM_SHARE *)
+      my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
+        &long_term_share, sizeof(*long_term_share),
+        &tmp_name, table_name_length + 1,
+        NullS))
+    ) {
+      *error = HA_ERR_OUT_OF_MEM;
+      goto error_alloc_long_term_share;
+    }
+    long_term_share->table_name = tmp_name;
+    long_term_share->table_name_length = table_name_length;
+    memcpy(long_term_share->table_name, table_name, table_name_length);
+    if (pthread_mutex_init(&long_term_share->auto_inc_mutex,
+                           MY_MUTEX_INIT_FAST))
+    {
+      *error = HA_ERR_OUT_OF_MEM;
+      goto error_init_auto_inc_mutex;
+    }
+    if (my_hash_insert(&mrn_long_term_share, (uchar*) long_term_share))
+    {
+      *error = HA_ERR_OUT_OF_MEM;
+      goto error_hash_insert;
+    }
+  }
+  DBUG_RETURN(long_term_share);
+
+error_hash_insert:
+  pthread_mutex_destroy(&long_term_share->auto_inc_mutex);
+error_init_auto_inc_mutex:
+  my_free(long_term_share, MYF(0));
+error_alloc_long_term_share:
+  DBUG_RETURN(NULL);
+}
+
 MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
 {
   MRN_SHARE *share;
@@ -749,7 +776,7 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
   TABLE_SHARE *wrap_table_share;
   MRN_DBUG_ENTER_FUNCTION();
   length = (uint) strlen(table_name);
-  pthread_mutex_lock(&mrn_open_tables_mutex);
+  mrn::Lock lock(&mrn_open_tables_mutex);
   if (!(share = (MRN_SHARE*) my_hash_search(&mrn_open_tables,
     (uchar*) table_name, length)))
   {
@@ -842,6 +869,11 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
       goto error_init_mutex;
     }
     thr_lock_init(&share->lock);
+    if (!(share->long_term_share = mrn_get_long_term_share(table_name, length,
+                                                           error)))
+    {
+      goto error_get_long_term_share;
+    }
     if (my_hash_insert(&mrn_open_tables, (uchar*) share))
     {
       *error = HA_ERR_OUT_OF_MEM;
@@ -849,24 +881,23 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
     }
   }
   share->use_count++;
-  pthread_mutex_unlock(&mrn_open_tables_mutex);
   DBUG_RETURN(share);
 
 error_hash_insert:
+error_get_long_term_share:
   pthread_mutex_destroy(&share->mutex);
 error_init_mutex:
 error_parse_table_param:
   mrn_free_share_alloc(share);
   my_free(share, MYF(0));
 error_alloc_share:
-  pthread_mutex_unlock(&mrn_open_tables_mutex);
   DBUG_RETURN(NULL);
 }
 
 int mrn_free_share(MRN_SHARE *share)
 {
   MRN_DBUG_ENTER_FUNCTION();
-  pthread_mutex_lock(&mrn_open_tables_mutex);
+  mrn::Lock lock(&mrn_open_tables_mutex);
   if (!--share->use_count)
   {
     my_hash_delete(&mrn_open_tables, (uchar*) share);
@@ -877,7 +908,6 @@ int mrn_free_share(MRN_SHARE *share)
     pthread_mutex_destroy(&share->mutex);
     my_free(share, MYF(0));
   }
-  pthread_mutex_unlock(&mrn_open_tables_mutex);
   DBUG_RETURN(0);
 }
 
@@ -885,9 +915,6 @@ TABLE_SHARE *mrn_get_table_share(TABLE_LIST *table_list, int *error)
 {
   uint key_length;
   TABLE_SHARE *share;
-#if MYSQL_VERSION_ID >= 50500
-  my_hash_value_type hash_value;
-#endif
   THD *thd = current_thd;
   MRN_DBUG_ENTER_FUNCTION();
 #if MYSQL_VERSION_ID >= 50603 && !defined(MRN_MARIADB_P)
@@ -895,12 +922,22 @@ TABLE_SHARE *mrn_get_table_share(TABLE_LIST *table_list, int *error)
   key_length = get_table_def_key(table_list, &key);
 #else
   char key[MAX_DBKEY_LENGTH];
+#  if MYSQL_VERSION_ID >= 100002 && defined(MRN_MARIADB_P)
+  key_length = create_table_def_key(key, table_list->db, table_list->table_name);
+#  else
   key_length = create_table_def_key(thd, key, table_list, FALSE);
+#  endif
 #endif
 #if MYSQL_VERSION_ID >= 50500
+  my_hash_value_type hash_value;
   hash_value = my_calc_hash(&table_def_cache, (uchar*) key, key_length);
+#  if MYSQL_VERSION_ID >= 100002 && defined(MRN_MARIADB_P)
+  share = get_table_share(thd, table_list->db, table_list->table_name, key,
+                          key_length, 0, hash_value);
+#  else
   share = get_table_share(thd, table_list, key, key_length, 0, error,
                           hash_value);
+#  endif
 #else
   share = get_table_share(thd, table_list, key, key_length, 0, error);
 #endif
@@ -922,10 +959,19 @@ TABLE_SHARE *mrn_create_tmp_table_share(TABLE_LIST *table_list, const char *path
   MRN_DBUG_ENTER_FUNCTION();
 #if MYSQL_VERSION_ID >= 50603 && !defined(MRN_MARIADB_P)
   key_length = get_table_def_key(table_list, &key);
+#elif MYSQL_VERSION_ID >= 100002 && defined(MRN_MARIADB_P)
+  key_length = create_table_def_key(key, table_list->db,
+                                    table_list->table_name);
 #else
   key_length = create_table_def_key(thd, key, table_list, FALSE);
 #endif
-  if (!(share = alloc_table_share(table_list, key, key_length)))
+#if MYSQL_VERSION_ID >= 100002 && defined(MRN_MARIADB_P)
+  share = alloc_table_share(table_list->db, table_list->table_name, key,
+                            key_length);
+#else
+  share = alloc_table_share(table_list, key, key_length);
+#endif
+  if (!share)
   {
     *error = ER_CANT_OPEN_FILE;
     DBUG_RETURN(NULL);
@@ -935,7 +981,7 @@ TABLE_SHARE *mrn_create_tmp_table_share(TABLE_LIST *table_list, const char *path
   share->path.length = strlen(path);
   share->normalized_path.str = share->path.str;
   share->normalized_path.length = share->path.length;
-  if (open_table_def(thd, share, 0))
+  if (open_table_def(thd, share, GTS_TABLE))
   {
     *error = ER_CANT_OPEN_FILE;
     DBUG_RETURN(NULL);
@@ -1006,14 +1052,14 @@ st_mrn_slot_data *mrn_get_slot_data(THD *thd, bool can_create)
     slot_data->alter_connect_string = NULL;
     slot_data->alter_comment = NULL;
     *thd_ha_data(thd, mrn_hton_ptr) = (void *) slot_data;
-    pthread_mutex_lock(&mrn_allocated_thds_mutex);
-    if (my_hash_insert(&mrn_allocated_thds, (uchar*) thd))
     {
-      pthread_mutex_unlock(&mrn_allocated_thds_mutex);
-      free(slot_data);
-      DBUG_RETURN(NULL);
+      mrn::Lock lock(&mrn_allocated_thds_mutex);
+      if (my_hash_insert(&mrn_allocated_thds, (uchar*) thd))
+      {
+        free(slot_data);
+        DBUG_RETURN(NULL);
+      }
     }
-    pthread_mutex_unlock(&mrn_allocated_thds_mutex);
   }
   DBUG_RETURN(slot_data);
 }
