@@ -44,6 +44,12 @@ fi
 
 . "${top_dir}/config.sh"
 
+bundled_groonga_normalizer_mysql_dir="${top_dir}/vendor/groonga/vendor/plugins/groonga-normalizer-mysql"
+if [ -d "${bundled_groonga_normalizer_mysql_dir}" ]; then
+    GRN_PLUGINS_DIR="${bundled_groonga_normalizer_mysql_dir}"
+    export GRN_PLUGINS_DIR
+fi
+
 source_mysql_test_dir="${MYSQL_SOURCE_DIR}/mysql-test"
 build_mysql_test_dir="${MYSQL_BUILD_DIR}/mysql-test"
 source_test_suites_dir="${source_mysql_test_dir}/suite"
@@ -68,7 +74,10 @@ case "${MYSQL_VERSION}" in
 	fi
 	if [ "${mariadb}" = "yes" ]; then
 	    if [ "${MRN_BUNDLED}" != "TRUE" ]; then
-		ln -s "${top_dir}" "${MYSQL_BUILD_DIR}/plugin/mroonga"
+		mariadb_mroonga_plugin_dir="${MYSQL_BUILD_DIR}/plugin/mroonga"
+		if [ ! -e "${mariadb_mroonga_plugin_dir}" ]; then
+		    ln -s "${top_dir}" "${mariadb_mroonga_plugin_dir}"
+		fi
 	    fi
 	    plugins_dir=
 	else
@@ -88,17 +97,11 @@ same_link_p()
     fi
 }
 
-local_mroonga_mysql_test_include_dir="${BASE_DIR}/sql/include"
-for test_include_name in $(ls $local_mroonga_mysql_test_include_dir | grep '\.inc$'); do
-    local_mroonga_mysql_test_include="${local_mroonga_mysql_test_include_dir}/${test_include_name}"
-    mroonga_mysql_test_include="${build_test_include_dir}/${test_include_name}"
-    if ! same_link_p "${local_mroonga_mysql_test_include}" \
-			"${mroonga_mysql_test_include}"; then
-	rm -f "${mroonga_mysql_test_include}"
-        ln -s "${local_mroonga_mysql_test_include}" \
-	    "${mroonga_mysql_test_include}"
-    fi
-done
+mroonga_mysql_test_include_dir="${build_test_include_dir}/mroonga"
+if ! same_link_p "${mroonga_mysql_test_include_dir}"; then
+    rm -f "${mroonga_mysql_test_include_dir}"
+    ln -s "${BASE_DIR}/sql/include/mroonga" "${mroonga_mysql_test_include_dir}"
+fi
 
 for test_suite_name in mroonga; do
     local_mroonga_mysql_test_suite_dir="${BASE_DIR}/sql/suite/${test_suite_name}"
@@ -135,7 +138,7 @@ if [ ! -d "${mroonga_wrapper_innodb_test_suite_dir}" ]; then
 	${mroonga_wrapper_innodb_test_suite_dir}/t/*.test \
 	${mroonga_wrapper_innodb_test_suite_dir}/include/*.inc
     sed -i'' \
-	-e '1 i --source include/have_mroonga.inc' \
+	-e '1 i --source include/mroonga/have_mroonga.inc' \
 	${mroonga_wrapper_innodb_test_suite_dir}/t/*.test
 fi
 
@@ -165,6 +168,10 @@ test_suite_names=""
 test_names=""
 while [ $# -gt 0 ]; do
     case "$1" in
+	--manual-gdb|--debug)
+	    n_processors=1
+	    break
+	    ;;
 	--*)
 	    break
 	    ;;
